/********************************************************************
 *                                                                  *
 * THIS FILE IS PART OF THE WAVCONFILTER SOURCE CODE.               *
 * USE, DISTRIBUTION AND REPRODUCTION OF THIS LIBRARY SOURCE IS     *
 * GOVERNED BY A BSD-STYLE SOURCE LICENSE INCLUDED WITH THIS SOURCE *
 * IN 'COPYING'. PLEASE READ THESE TERMS BEFORE DISTRIBUTING.       *
 *                                                                  *
 * THE WAVCONFILTER SOURCE CODE IS (C) COPYRIGHT 2008 Cocha         *
 * http://sourceforge.jp/projects/directshow/                       *
 *                                                                  *
 ********************************************************************/

#ifndef __WavConFilter_h__
#define __WavConFilter_h__

#include <windows.h>
#include <streams.h>
#include <math.h>

#define WAVE_FORMAT_IEEE_FLOAT 0x0003

#define SAFE_RELEASE(p) { if(p!=NULL) { (p)->Release(); (p)=NULL; } }
struct SmplData{double l,r;};


// tB^̖O
#define FILTER_NAME L"WavConFilter"


// {EFBF44B7-BB1F-48c8-BA59-EB9803579B1E}
static const GUID CLSID_WavConFilter = 
{ 0xefbf44b7, 0xbb1f, 0x48c8, { 0xba, 0x59, 0xeb, 0x98, 0x3, 0x57, 0x9b, 0x1e } };

// -----------------------------------------------------------------------------------------------------------------------------------
class CSamplingConv
{
public:

   // [U֐
   HRESULT ConvertInit(WAVEFORMATEX *, WAVEFORMATEX *, int);
   void Convert(struct SmplData *, int, BYTE *, int *);
   void ConvertEnd(struct SmplData *, int, BYTE *, int *);

private:

   // [U֐
   double bessel0(double);

   // [Uϐ
   WAVEFORMATEX m_inFormat;
   WAVEFORMATEX m_outFormat;

   int iptr;
	int ismd, ismr;
   long firodrv;
   int isrm, osrm;
   int sptr, sptrr, eptr, eptrr;
   double m_hfir[65536];
   double m_dMaxData, m_dMinData;
   double m_dRatioPlus, m_dRatioMinus;
};
// -----------------------------------------------------------------------------------------------------------------------------------
class CChannelConv
{
public:

   // [U֐
   HRESULT SetFormat(WAVEFORMATEX *, WAVEFORMATEX *);
   HRESULT Convert(BYTE *, int, BYTE *, int *);
   HRESULT Convert(BYTE *, int, int *);

private:

   // [Uϐ
   WAVEFORMATEX m_inFormat;
   WAVEFORMATEX m_outFormat;

};
// -----------------------------------------------------------------------------------------------------------------------------------
class CBitConv
{
public:

   // [U֐
   HRESULT SetFormat(WAVEFORMATEX *, WAVEFORMATEX *);
   HRESULT Convert(BYTE *, int, BYTE *, int *);

private:

   // [Uϐ
   WAVEFORMATEX m_inFormat;
   WAVEFORMATEX m_outFormat;
   double m_dRatio;

   // [U֐
   HRESULT Convert8(BYTE *, int, BYTE *, int *);
   HRESULT Convert16(BYTE *, int, BYTE *, int *);
   HRESULT Convert24(BYTE *, int, BYTE *, int *);
   HRESULT Convert32i(BYTE *, int, BYTE *, int *);
   HRESULT Convert32f(float *, int, BYTE *, int *);
   HRESULT Convert64(double *, int, BYTE *, int *);
};
// -----------------------------------------------------------------------------------------------------------------------------------
class CWavConFilter : public CTransformFilter
{
   friend class CSamplingConv;
   friend class CChannelConv;
   friend class CBitConv;

public:
   DECLARE_IUNKNOWN

   // RXgN^ƃfXgN^
   CWavConFilter(LPUNKNOWN pUnk,HRESULT *phr);
   virtual ~CWavConFilter();

   // CTransformFilteȑz\bh
   HRESULT CheckInputType(const CMediaType *);
   HRESULT CheckTransform(const CMediaType *,const CMediaType *);
   HRESULT GetMediaType(int iPosition,CMediaType *);
   HRESULT DecideBufferSize(IMemAllocator *, ALLOCATOR_PROPERTIES *);

   // CTransformFilter̃I[o[Ch
   HRESULT StartStreaming(void);
   HRESULT Receive(IMediaSample *);
   HRESULT StopStreaming();
   HRESULT EndOfStream(void);

   // IWavConInterface
   STDMETHODIMP GetInFormat(WAVEFORMATEX *);
   STDMETHODIMP GetOutFormat(WAVEFORMATEX *);
   STDMETHODIMP SetOutFormat(WAVEFORMATEX *);
   STDMETHODIMP SetQuality(int);

protected:

private:

   // [U֐
   void ConvertToDouble(BYTE *, int);

   // [Uϐ
   bool m_bSetOutFormat;
   int m_nAccurate;
   WAVEFORMATEX m_inFormat;
   WAVEFORMATEX m_outFormat;
   CBitConv *m_pBitConv;
   CCritSec m_myLock;
   CChannelConv *m_pChannelConv;
   CSamplingConv *m_pSamplingConv;
   int m_nSmplSize;
   struct SmplData *m_smpld;
};

#endif __WavConFilter_h__
